/***************************************************************************
* File Name: hal430_RS232Tx.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2013 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2013Nov10	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions initialize and broadcast formatted data
* to the UART pins configured for this application note.
* 	- Configure UART for 38.4kBaud based on a ~4.9MHz DCO frequency
* 	- Broadcast characters or strings of characters the the UART
* 	- Format and broadcast temperature data from the MAX31855K-4CH board
***************************************************************************/

#include "msp430x16x.h"
#include "hal430_RS232Tx.h"
#include "stdio.h"
#include "pwf_an0002_defines.h"
#include "msp430_typeDef.h"
#include "pwf_an0002_struct.h"

extern struct	var_MAX31865_ch temp_RTD0;

// character strings for UART broadcast headings
static char RTD_TEMP0_STR[] 		= " RTD Temp: ";
static char RTD_RES0_STR[] 			= "RTD Res: ";
static char RTD_CONF_STR[]			= " CONF:  ";
static char RTD_STAT_STR[]			= " STAT:  ";
static char DEC_PT[]				= ".";

void RS232_Tx_init(void)	// Modified for use on Oven Controller V2.0
{
	// UART Clock set to SMCLK(DCO = 4.9 MHz)
	U1CTL = SWRST;								// U1CTL SWRST 
	ME2 |= UTXE1;								// Enable USART1 TXD
	U1CTL |= CHAR;								// 8-bit character
	U1TCTL = SSEL0 + SSEL1;						// BRCLK = SMCLK
	U1BR0 = 0x7F;								// ~4.9 MHz SMCLK/ 127.6  = 38400 baud
	U1BR1 = 0x00;								//
	U1MCTL = 0xB5;								// Modulation for 4.9 MHz SMCLK, 38400 baud
	U1CTL &= ~SWRST;							// Initialize USART state machine
}

void RS232_putchar(unsigned char data)
{
	while (!(U1IFG & UTXIFG1));                  // wait for Tx buffer to be empty - transmit when Tx buffer empty
	U1TXBUF = data;
}

void RS232_putCharStr(char *data)
{
  while(*data)
  {
	RS232_putchar(*data++);
  }
}

void RS232_disable(void)
{
  while(0x01 & ~U1TCTL)                     // wait until TXEPT is set (1)
  UCTL1 = SWRST;
}

void RS232_return(void)
{
  while(!(U1IFG & UTXIFG1));
  U1TXBUF = 0x0D;
}

void RS232_tab(void)
{
  while(!(U1IFG & UTXIFG1));
  U1TXBUF = 0x09;
}

void uart_single_ch_write(struct var_MAX31865_ch *RTD_ptr)
{
	uint16 size, frac16;
	uint32 frac32;
	char conv_to_str[8];
	
	// RTD0
	frac32 = RTD_ptr->rtd_res>>15;
	frac16 = (uint16)frac32;
	RS232_putCharStr(RTD_RES0_STR);
	size = sprintf(conv_to_str, "%d", (frac16));		// RTD resistance DECIMAL to string
	RS232_putCharStr(conv_to_str);						// Write decimal portion of resistance to UART (char string)
	RS232_putCharStr(DEC_PT);							// Write the decimal point
	frac16 = ((uint16)RTD_ptr->rtd_res & 0x7FFF)>>12;		// separate out fraction portion, reduce to 3 bits
	frac16 = frac16*125;								// multiply by 125 to give rwv conversion
	if(0 == frac16){RS232_putCharStr("000");}			// write triple zero if 'zero' in fraction
	else{
	size = sprintf(conv_to_str, "%d", (frac16));		// RTD resistance FRACTION to string
	RS232_putCharStr(conv_to_str);}						// Write fraction portion of resistance to UART (char string)
	RS232_putCharStr("ohm");							// write 'ohm'
	
	RS232_putCharStr(RTD_TEMP0_STR);
	size = sprintf(conv_to_str, "%d", RTD_ptr->rtd_temp>>5);	// RTD temperature DECIMAL to string
	RS232_putCharStr(conv_to_str);								// Write decimal portion of temp to UART (char string)
	RS232_putCharStr(DEC_PT);									// Write the decimal point
	frac16 = (RTD_ptr->rtd_temp & 0x1F)>>2;						// separate out fraction portion, reduce to 3 bits
	frac16 = frac16*125;										// multiply by 125 to give rwv conversion
	if(0 == frac16){RS232_putCharStr("000");}					// write triple zero if 'zero' in fraction
	else{
	size = sprintf(conv_to_str, "%d", (frac16));				// RTD resistance FRACTION to string
	RS232_putCharStr(conv_to_str);}								// Write fraction portion of resistance to UART (char string)
	RS232_putCharStr("degC");									// write deg C
	
	RS232_putCharStr(RTD_CONF_STR);
	size = sprintf(conv_to_str, "%d", RTD_ptr->config);	// RTD configuration to string
	RS232_putCharStr(conv_to_str);			// Write decimal portion of temp to UART (char string)
	
	RS232_putCharStr(RTD_STAT_STR);
	size = sprintf(conv_to_str, "%d", RTD_ptr->status);	// RTD status to string
	RS232_putCharStr(conv_to_str);			// Write decimal portion of temp to UART (char string)
	
	RS232_putCharStr(" HFT_REG: ");
	size = sprintf(conv_to_str, "%d", RTD_ptr->HFT_val);	// RTD HFT reg to string
	RS232_putCharStr(conv_to_str);			// Write decimal portion of temp to UART (char string)
	
	RS232_putCharStr(" LFT_REG: ");
	size = sprintf(conv_to_str, "%d", RTD_ptr->LFT_val);	// RTD LFT reg to string
	RS232_putCharStr(conv_to_str);			// Write decimal portion of temp to UART (char string)
}

void update_uart_250ms(void)
{
	// Broadcast temperatures to UART with headings formatted for direct
	// reading via 'hyperterminal' or 'putty'
	struct var_MAX31865_ch *RTD_ch_ptr;
	RTD_ch_ptr = &temp_RTD0;
	uart_single_ch_write(RTD_ch_ptr);
	
	RS232_return();						// start next line

}
