/***************************************************************************
* File Name: hal430_spi.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2013 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2013Nov30	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions to set the USART used in this application 
* note based on a ~4.9MHz DCO frequency and to ensure communications don't 
* overlap.
***************************************************************************/

#include "msp430x16x.h"
#include "msp430_typeDef.h"
#include "hal430_spi.h"
#include "pwf_an0002_defines.h"


void HalSpiInit_an0002(void)
{
	// SPI is on USART0... Set up to be below max frequency of MAX31855K chip. ~1.225MHz works
	U0CTL  = SWRST;							// U0CTL in SWRST state
	U0CTL  = CHAR + SYNC + MM + SWRST;		// USART0 module operation
											// CHAR = 1 => 8-bit data 
											// SYNC = 1 => SPI selected
											// MM = 1 => master mode,
											// MSP is the master
	
	U0TCTL = SSEL0 + SSEL1 + STC + CKPL;// + CKPH;	// SMCLK - ~4.9 MHz (see hal430_clocks_430f1xx.c)
                                      		// SSEL0 = 1 & SSEL1 = 1 sets SMCLK as clock source
                                     		// STC = 1 => 3-pin SPI mode selected
											// CKPH = 1 => read data on first rising edge
											// CKPL = 0 => clock starts low
	
	U0BR0  = 0x08;							// BRCLK = SMCLK/(UxBR0+UxBR1) = ~2.45MHz, max_bd_rt = BRCLK/2 = 1.225MHz
												// x02 -> 1.225 MHz
												// x04 -> 612.5 kHz
												// x08 -> 306.25 kHz
	U0BR1  = 0x00;
	U0MCTL = 0x00;							// Modulation control - not used for SPI mode. Ensure 
											// all these bits are reset for SPI mode
	
	ME1   |= USPIE0;						// Enable SPI Module
	U0CTL &= ~SWRST;						// SWRST clear
	
	// Wait a few clocks for changes to take effect
	volatile unsigned int i;
	for(i=0; i<10000; i++)
	{}
}

void complete(void)
{
	do{IFG1 &=~ URXIFG0;}					// clear RX interrupt flag in case its full
	while(!(IFG1 & UTXIFG0));				// wait for TX register to be empty
	while(!(URXIFG0 & IFG1));      			// wait for RX register to receive complete character
}   
